package com.Vente;

import java.util.ArrayList;
import java.util.List;

/**
 * Classe Panier : représente un panier contenant plusieurs produits.
 * Vérifie l'invariant : la liste de produits ne doit jamais être null.
 */
public class Panier {
    private List<Produit> produits;

    public Panier() {
        this.produits = new ArrayList<>();
        assert checkInvariant() : "Invariant violé : La liste de produits ne doit jamais être null.";
    }

    /**
     * Ajoute un produit au panier après vérification de la validité des paramètres.
     * @param quantite Nombre d'unités à ajouter (> 0).
     * @param chose Objet vendable (ex : Disque, Nourriture).
     */
    public void add(int quantite, Vendable chose) {
        if (quantite <= 0) {
            throw new IllegalArgumentException("Quantité invalide.");
        }
        produits.add(new Produit(chose, quantite));

        assert checkInvariant() : "Invariant violé : Panier invalide après ajout.";
    }

    /**
     * Calcule le prix total du panier en additionnant les prix de tous les produits.
     * @return Prix total du panier.
     */
    public double calculerTotal() {
        double total = produits.stream().mapToDouble(Produit::getPrixTotal).sum();
        assert total >= 0 : "Postcondition violée : Le prix total ne peut pas être négatif.";
        return total;
    }

    /**
     * Vérifie que l'invariant du panier est respecté.
     * @return true si la liste de produits est valide.
     */
    private boolean checkInvariant() {
        return produits != null && produits.stream().allMatch(p -> p.getPrixTotal() >= 0);
    }
}
